@@ -1,8 +1,9 @@
 #
 # -*- Perl -*-
-# $Id: olemsword.pl,v 1.8.4.4 2002/07/24 10:44:26 baba Exp $
-# Copyright (C) 1999 Jun Kurabe ,
-#		1999-2000 Ken-ichi Hirose All rights reserved.
+# $Id: olemsword.pl,v 1.8.4.18 2007/01/18 08:51:03 opengl2772 Exp $
+# Copyright (C) 1999 Jun Kurabe,
+#		1999-2000 Ken-ichi Hirose,
+#               2004-2007 Namazu Project All rights reserved.
 #     This is free software with ABSOLUTELY NO WARRANTY.
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -51,18 +52,57 @@ no strict 'refs';  # for symbolic reference: $fields;
 require 'util.pl';
 require 'gfilter.pl';
 
+my $version = 0;
+
 sub mediatype() {
-    return ('application/msword');
+    status();
+
+    if ($version >= 9) {
+        # 12.0 Office 2007
+        # 11.0 Office 2003
+        # 10.0 Office 2002
+        #  9.0 Office 2000
+        return (
+            'application/msword',
+            'application/vnd.openxmlformats-officedocument.wordprocessingml',
+        );
+    } else {
+        #  8.0 Office 97, 98
+        return (
+            'application/msword',
+        );
+    }
 }
 
 sub status() {
+    my $const = undef;
+
     open (SAVEERR,">&STDERR");
     open (STDERR,">nul");
-    my $const;
-    $const = Win32::OLE::Const->Load("Microsoft Word 10.0 Object Library");
-    $const = Win32::OLE::Const->Load("Microsoft Word 9.0 Object Library") unless $const;
-    $const = Win32::OLE::Const->Load("Microsoft Word 8.0 Object Library") unless $const;
+
+    if (!defined $const) {
+        $const = Win32::OLE::Const->Load("Microsoft Word 12.0 Object Library");
+        $version = 12;
+    }
+    if (!defined $const) {
+        $const = Win32::OLE::Const->Load("Microsoft Word 11.0 Object Library");
+        $version = 11;
+    }
+    if (!defined $const) {
+        $const = Win32::OLE::Const->Load("Microsoft Word 10.0 Object Library");
+        $version = 10;
+    }
+    if (!defined $const) {
+        $const = Win32::OLE::Const->Load("Microsoft Word 9.0 Object Library");
+        $version = 9;
+    }
+    if (!defined $const) {
+        $const = Win32::OLE::Const->Load("Microsoft Word 8.0 Object Library");
+        $version = 8;
+    }
+
     open (STDERR,">&SAVEERR");
+
     return 'yes' if (defined $const);
     return 'no';
 }
@@ -76,13 +116,15 @@ sub pre_codeconv() {
 }
 
 sub post_codeconv () {
-    return 1;
+    return 0;
 }
 
 sub add_magic ($) {
     my ($magic) = @_;
 
+    # FIXME: very ad hoc.
     $magic->addFileExts('\\.doc$', 'application/msword');
+    $magic->addFileExts('\\.docx$', 'application/vnd.openxmlformats-officedocument.wordprocessingml');
     return;
 }
 
@@ -92,10 +134,8 @@ sub filter ($$$$$) {
 
     util::vprint("Processing ms-word file ... (using  'Win32::OLE->new Word.Application')\n");
 
-    $cfile =~ s/\//\\/g;
-    $$cont = "";
-    ReadMSWord::ReadMSWord($cfile, $cont, $fields, $weighted_str);
-    $cfile = defined $orig_cfile ? $$orig_cfile : '';
+    my $err = ReadMSWord::ReadDocument($cfile, $cont, $fields, $weighted_str);
+    return $err if (defined $err);
 
     gfilter::line_adjust_filter($cont);
     gfilter::line_adjust_filter($weighted_str);
@@ -134,27 +174,41 @@ sub getProperties ($$$) {
     my $title = $cfile->BuiltInDocumentProperties('Title')->{Value};
     $title = $cfile->BuiltInDocumentProperties('Subject')->{Value}
 	unless (defined $title);
-    $fields->{'title'} = codeconv::shiftjis_to_eucjp($title)
-	if (defined $title);
+    if (defined $title) {
+        $title = codeconv::shiftjis_to_eucjp($title);
+        codeconv::normalize_eucjp_document(\$title);
+        $fields->{'title'} = $title;
 
-    my $weight = $conf::Weight{'html'}->{'title'};
-    $$weighted_str .= "\x7f$weight\x7f$fields->{'title'}\x7f/$weight\x7f\n";
+        my $weight = $conf::Weight{'html'}->{'title'};
+        $$weighted_str .= "\x7f$weight\x7f$fields->{'title'}\x7f/$weight\x7f\n";
+    }
 
     my $author = $cfile->BuiltInDocumentProperties('Last Author')->{Value};
     $author = $cfile->BuiltInDocumentProperties('Author')->{Value}
 	unless (defined $author);
-    $fields->{'author'} = codeconv::shiftjis_to_eucjp($author)
-	if (defined $author);
+    if (defined $author) {
+        $author = codeconv::shiftjis_to_eucjp($author);
+        codeconv::normalize_eucjp_document(\$author);
+        $fields->{'author'} = $author;
+    }
 
-#    my $date = $cfile->BuiltInDocumentProperties('Last Save Time')->{Value};
-#    $date = $cfile->BuiltInDocumentProperties('Creation Date')->{Value}
-#	unless (defined $date);
-#    $fields->{'date'} = codeconv::shiftjis_to_eucjp($date) if (defined $date);
+    # my $date = $cfile->BuiltInDocumentProperties('Last Save Time')->{Value};
+    # $date = $cfile->BuiltInDocumentProperties('Creation Date')->{Value}
+    #    unless (defined $date);
+    # if (defined $date) {
+    #     $date = codeconv::shiftjis_to_eucjp($date);
+    #     codeconv::normalize_eucjp_document(\$date);
+    #     $fields->{'date'} = $date;
+    # }
 
     my $keyword = $cfile->BuiltInDocumentProperties('keywords')->{Value};
-    $keyword = codeconv::shiftjis_to_eucjp($keyword);
-    $weight = $conf::Weight{'metakey'};
-    $$weighted_str .= "\x7f$weight\x7f$keyword\x7f/$weight\x7f\n";
+    if (defined $keyword) {
+        $keyword = codeconv::shiftjis_to_eucjp($keyword);
+        codeconv::normalize_eucjp_document(\$keyword);
+
+        my $weight = $conf::Weight{'metakey'};
+        $$weighted_str .= "\x7f$weight\x7f$keyword\x7f/$weight\x7f\n";
+    }
 
     return undef;
 }
@@ -164,38 +218,91 @@ package ReadMSWord;
 my $word = undef;
 my $office_consts = undef;
 
+# Word application destructor
+END {
+    if (defined $word) {
+	util::vprint("Word->Quit\n");
+	$word->Quit(0);
+	undef $word;
+    }
+}
+
+sub GetExt($) {
+    my ($filename) = @_;
+
+    my $ext = $filename;
+    $ext =~ s!.*/!!g;
+    if ($ext !~ s/^.*(\.[^\.]*)$/$1/) {
+        $ext = "";
+    }
+
+    return $ext;
+}
+
+sub ReadDocument ($$$$) {
+    my ($cfile, $cont, $fields, $weighted_str) = @_;
+
+    my $ext = GetExt($cfile);
+    my $tmpfile = util::tmpnam('NMZ.olemsword') . $ext;
+    {
+        my $fh = util::efopen("> $tmpfile");
+        print $fh $$cont;
+        util::fclose($fh);
+    }
+
+    my $err = ReadMSWord::ReadMSWord($tmpfile, $cont, $fields, $weighted_str);
+    unlink $tmpfile;
+
+    # codeconv::toeuc($cont);
+    codeconv::codeconv_document($cont);
+
+    return $err;
+}
+
 sub ReadMSWord ($$$$) {
     my ($cfile, $cont, $fields, $weighted_str) = @_;
 
+    $cfile =~ s/\//\\/g;
+    $$cont = "";
+
     # Copy From Win32::OLE Example Program
     # use existing instance if Word is already running
-    eval {$word = Win32::OLE->GetActiveObject('Word.Application')};
-    die "MSWord not installed" if $@;
+    # eval {$word = Win32::OLE->GetActiveObject('Word.Application')};
+    # die "MSWord not installed" if $@;
     unless (defined $word) {
-	$word = Win32::OLE->new('Word.Application')
-	    or die "Oops, cannot start Word";
+	$word = Win32::OLE->new('Word.Application', 'Quit')
+	    or return "Oops, cannot start Word." . Win32::OLE->LastError();
     }
     # End of Copy From Win32::OLE Example Program
 
     # Redirect stderr to null device, to ignore Error and Exception message.
     open (SAVEERR,">&STDERR");
     open (STDERR,">nul");
-    # Load Office 97/98/2000/XP Constant
-    $office_consts = Win32::OLE::Const->Load("Microsoft Office 10.0 Object Library");
+    # Load Office 97/98/2000/XP/2003/2007 Constant
+    $office_consts = Win32::OLE::Const->Load("Microsoft Office 12.0 Object Library") unless $office_consts;
+    $office_consts = Win32::OLE::Const->Load("Microsoft Office 11.0 Object Library") unless $office_consts;
+    $office_consts = Win32::OLE::Const->Load("Microsoft Office 10.0 Object Library") unless $office_consts;
     $office_consts = Win32::OLE::Const->Load("Microsoft Office 9.0 Object Library") unless $office_consts;
     $office_consts = Win32::OLE::Const->Load("Microsoft Office 8.0 Object Library") unless $office_consts;
+
+    $Win32::OLE::Warn = 0;            # False
     # for debug
-    # $word->{Visible} = 1;
+    # $word->{Visible} = 1;             # True
+    $word->{Visible} = 0;             # False
+    $word->{DisplayAlerts} = 0;       # wdAlertsNone
+    $word->{AutomationSecurity} = 3;  # msoAutomationSecurityForceDisable
+
     # Restore stderr device usually.
     open (STDERR,">&SAVEERR");
 
     # In order to skip password-protected file, send a dummy password.
-    my $doc = $word->{Documents}->open({
-	'FileName' => $cfile,
+    my $doc = $word->{Documents}->Open({
+	'FileName'         => $cfile,
 	'PasswordDocument' => 'dummy password',
-	'ReadOnly' => 1
+	'PasswordTemplate' => 'dummy password',
+	'ReadOnly'         => 1,  # True
 	});
-    die "Cannot open File $cfile" unless (defined $doc) ;
+    return "Cannot open File $cfile" unless (defined $doc);
 
     olemsword::getProperties($doc, $fields, $weighted_str);
     getParagraphs($doc, $cont);
@@ -203,20 +310,14 @@ sub ReadMSWord ($$$$) {
     getShapes($doc, $cont);
     getHeadersFooters($doc, $cont);
 
-    $doc->close(0);
+    $doc->Close({
+        'SaveChanges' => 0  # False
+    });
     undef $doc;
 
     return undef;
 }
 
-END {
-    if (defined $word) {
-	util::vprint("Word->Quit\n");
-	$word->Quit;
-	undef $word;
-    }
-}
-
 sub getParagraphs ($$) {
     my ($doc, $cont) = @_;
 
@@ -257,6 +358,7 @@ sub getFrames ($$) {
 
     my $enum_func = sub {
 	my $obj = shift;
+        return 1 if (!defined($obj->Range));
 	my $p = $obj->Range->{Text};
 #	chop $p;
         $$cont .= "$p\n" if ( defined $p );
