@@ -1,9 +1,9 @@
 /*
  * 
- * $Id: field.c,v 1.29.8.9 2005/07/21 08:47:54 opengl2772 Exp $
+ * $Id: field.c,v 1.29.8.10 2006/04/15 17:44:18 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2004 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2006 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -122,7 +122,7 @@ nmz_isfield(const char *key)
         }
         key++;
     }
-    if (isalpha((unsigned char)*(key - 1)) && *key == ':' ) {
+    if (nmz_isalpha((unsigned char)*(key - 1)) && *key == ':' ) {
         return 1;
     }
     return 0;
