@@ -2,10 +2,10 @@
  * 
  * hlist.c -
  * 
- * $Id: hlist.c,v 1.51.4.3 2001/09/08 07:42:02 takesako Exp $
+ * $Id: hlist.c,v 1.51.4.20 2006/09/23 08:26:44 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2006 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -114,6 +114,11 @@ field_sort(NmzResult hlist)
 
 	hlist.data[i].field = malloc(leng + 1);
 	if (hlist.data[i].field == NULL) {
+            int j;
+            for (j = 0; j < i; j++) {
+	        free(hlist.data[j].field);
+	        hlist.data[j].field = NULL;
+            }
 	    nmz_set_dyingmsg(nmz_msg("%s", strerror(errno)));
 	    return FAILURE;
 	}
@@ -129,6 +134,7 @@ field_sort(NmzResult hlist)
 
     for (i = 0; i < hlist.num; i++) {
 	free(hlist.data[i].field);
+	hlist.data[i].field = NULL;
     }
     return 0;
 }
@@ -273,7 +279,10 @@ nmz_andmerge(NmzResult left, NmzResult right, int *ignore)
 		break;
 	    if (left.data[i].docid == right.data[j].docid) {
 
-		nmz_copy_hlist(left, v, left, i);
+                if (v != i) {
+                    nmz_copy_hlist(left, v, left, i);
+                }
+
                 if (nmz_is_tfidfmode()) {
                     left.data[v].score = 
 			left.data[i].score + right.data[j].score;
@@ -335,7 +344,9 @@ nmz_notmerge(NmzResult left, NmzResult right, int *ignore)
 	    }
 	}
 	if (!f) {
-	    nmz_copy_hlist(left, v, left, i);
+            if (v != i) {
+	        nmz_copy_hlist(left, v, left, i);
+            }
 	    v++;
 	}
     }
@@ -356,6 +367,10 @@ nmz_ormerge(NmzResult left, NmzResult right)
     int i, j, v, n;
     NmzResult val;
 
+    val.num  = 0;
+    val.data = NULL;
+    val.stat = SUCCESS;
+
     if ((left.stat  != SUCCESS || left.num <= 0) && 
 	(right.stat != SUCCESS || right.num <= 0)) 
     {
@@ -374,12 +389,17 @@ nmz_ormerge(NmzResult left, NmzResult right)
     n = left.num + right.num;
 
     nmz_malloc_hlist(&val, n);
-    if (val.stat == ERR_FATAL)
+    if (val.stat == ERR_FATAL) {
+	nmz_free_hlist(left);
+	nmz_free_hlist(right);
         return val;
+    }
 
     for (v = 0, i = 0, j = 0; i < left.num; i++) {
-	for (; left.data[i].docid >= right.data[j].docid && j < right.num; j++) {
-	    if (left.data[i].docid == right.data[j].docid) {
+	for (; j < right.num; j++) {
+	    if (left.data[i].docid < right.data[j].docid) {
+                break;
+            } else if (left.data[i].docid == right.data[j].docid) {
 
                 if (nmz_is_tfidfmode()) {
                     left.data[i].score = 
@@ -415,6 +435,10 @@ nmz_ormerge(NmzResult left, NmzResult right)
 void 
 nmz_malloc_hlist(NmzResult * hlist, int n)
 {
+    hlist->num  = 0;
+    hlist->data = NULL;
+    hlist->stat = SUCCESS;
+
     if (n <= 0) return;
     hlist->data = malloc(n * sizeof(struct nmz_data));
     if (hlist->data == NULL) {
@@ -466,6 +490,10 @@ nmz_merge_hlist(NmzResult *hlists)
     int i, n;
     NmzResult value;
 
+    value.num  = 0;
+    value.data = NULL;
+    value.stat = SUCCESS;
+
     if (nmz_get_idxnum() == 1) {
 	return hlists[0];
     }
@@ -476,8 +504,15 @@ nmz_merge_hlist(NmzResult *hlists)
         }
     }
     nmz_malloc_hlist(&value, n);
-    if (value.stat == ERR_FATAL)
+    if (value.stat == ERR_FATAL) {
+        for(i = 0; i < nmz_get_idxnum(); i++) {
+            if (hlists[i].stat != SUCCESS || hlists[i].num <= 0)
+                continue;
+            nmz_free_hlist(hlists[i]);
+        }
         return value;
+    }
+
     for(i = n = 0; i < nmz_get_idxnum(); i++) {
         if (hlists[i].stat != SUCCESS || hlists[i].num <= 0) 
             continue;
@@ -497,7 +532,7 @@ NmzResult
 nmz_do_date_processing(NmzResult hlist)
 {
     FILE *date_index;
-    int i;
+    int i, v;
 
     date_index = fopen(NMZ.t, "rb");
     if (date_index == NULL) {
@@ -506,30 +541,33 @@ nmz_do_date_processing(NmzResult hlist)
         return hlist; /* error */
     }
 
-    for (i = 0; i < hlist.num ; i++) {
-        if (-1 == fseek(date_index, 
-			hlist.data[i].docid * sizeof(hlist.data[i].date), 0)) 
+    for (i = 0; i < hlist.num; i++) {
+        if (fseek(date_index,
+                hlist.data[i].docid * sizeof(hlist.data[i].date), 0) != 0)
 	{
 	    nmz_set_dyingmsg(nmz_msg("%s: %s", NMZ.t, strerror(errno)));
 	    hlist.stat = ERR_FATAL;
+            fclose(date_index);
             return hlist; /* error */
         }
         nmz_fread(&hlist.data[i].date, 
 		  sizeof(hlist.data[i].date), 1, date_index);
-
-        if (hlist.data[i].date == -1) {  
-            /* The missing number, this document has been deleted */
-            int j;
-
-            for (j = i + 1; j < hlist.num; j++) { /* shift */
-                nmz_copy_hlist(hlist, j - 1, hlist, j);
-            }
-            hlist.num--;
-            i--;
-        }
     }
 
     fclose(date_index);
+
+    for (v = 0, i = 0; i < hlist.num; i++) {
+        if (hlist.data[i].date == -1) {  
+            /* The missing number, this document has been deleted */
+        } else {
+            if (v != i) {
+                nmz_copy_hlist(hlist, v, hlist, i);
+            }
+            v++;
+        }
+    }
+    hlist.num = v;
+
     return hlist;
 }
 
@@ -539,45 +577,58 @@ nmz_do_date_processing(NmzResult hlist)
 NmzResult 
 nmz_get_hlist(int index)
 {
-    int n, *buf, i;
     NmzResult hlist;
-    double idf = 1.0;
 
     hlist.num  = 0;
+    hlist.data = NULL;
     hlist.stat = SUCCESS;
 
-    if (-1 == fseek(Nmz.i, nmz_getidxptr(Nmz.ii, index), 0)) {
+    if (fseek(Nmz.i, nmz_getidxptr(Nmz.ii, index), 0) != 0) {
 	hlist.stat = ERR_FATAL;
 	return hlist; /* error */
     }
 
-    nmz_get_unpackw(Nmz.i, &n);
-
-    if (nmz_is_tfidfmode() &&
-	(nmz_get_querytokennum() > 1
-	 /* 0th token is a phrase. */
-	 || strchr(nmz_get_querytoken(0), '\t') != NULL)) 
     {
-        idf = log((double)document_number / (n/2)) / log(2);
-	nmz_debug_printf("idf: %f (N:%d, n:%d)\n", idf, document_number, n/2);
-    }
-
-    if (n > nmz_get_maxhit() * 2) {  
-        /* '* 2' means NMZ.i contains a file-ID and a score. */
-        hlist.stat = ERR_TOO_MUCH_HIT;
-    } else {
+        int n, *buf, i;
+        double idf = 1.0;
 	int sum = 0;
-	buf = malloc(n * sizeof(int)); /* with pelnty margin */
+	int hit;
+	int bersize;
+	int totalsize;
+
+        nmz_get_unpackw(Nmz.i, &bersize);
+
+	hit = bersize;
+	buf = malloc(hit * sizeof(int));
 	if (buf == NULL) {
 	    nmz_set_dyingmsg(nmz_msg("%s", strerror(errno)));
 	    hlist.data = NULL;
 	    hlist.stat = ERR_FATAL;
 	    return hlist;
 	}
-	n = nmz_read_unpackw(Nmz.i, buf, n) / 2;
+
+        n = 0;
+        totalsize = 0;
+        while (totalsize < bersize) {
+            totalsize += nmz_get_unpackw(Nmz.i, &buf[n]);
+            n++;
+        }
+        n /= 2;
+
+        if (nmz_is_tfidfmode() &&
+	    (nmz_get_querytokennum() > 1
+	     /* 0th token is a phrase. */
+	     || strchr(nmz_get_querytoken(0), '\t') != NULL)) 
+        {
+            idf = log((double)document_number / n) / log(2);
+	    nmz_debug_printf("idf: %f (N:%d, n:%d)\n", idf, document_number, n);
+        }
+
 	nmz_malloc_hlist(&hlist, n);
-	if (hlist.stat == ERR_FATAL)
+	if (hlist.stat == ERR_FATAL) {
+	    free(buf);
 	    return hlist;
+        }
 	
 	for (i = 0; i < n; i++) {
 	    hlist.data[i].docid = *(buf + i * 2) + sum;
@@ -622,21 +673,20 @@ int
 nmz_reverse_hlist(NmzResult hlist)
 {
     int m, n;
-    NmzResult tmp;
+    struct nmz_data tmp;
 
-    nmz_malloc_hlist(&tmp, 1);
-    if (tmp.stat == ERR_FATAL)
-        return FAILURE;
     m = 0;
     n = hlist.num - 1;
     while (m < n) {
-	nmz_copy_hlist(tmp, 0, hlist, m);
-	nmz_copy_hlist(hlist, m, hlist, n);
-	nmz_copy_hlist(hlist, n, tmp, 0);
-	m++;
-	n--;
+        /* swap */
+        tmp = hlist.data[m];
+        hlist.data[m] = hlist.data[n];
+        hlist.data[n] = tmp;
+
+        m++;
+        n--;
     }
-    nmz_free_hlist(tmp);
+
     return 0;
 }
 
@@ -649,7 +699,8 @@ nmz_set_docnum(int n)
 void 
 nmz_set_sortfield(const char *field)
 {
-    strcpy(field_for_sort, field);
+    strncpy(field_for_sort, field, BUFSIZE - 1);
+    field_for_sort[BUFSIZE - 1] = '\0';
 }
 
 char *
