@@ -2,10 +2,10 @@
  * 
  * score.c - scoring-related functions of Namazu
  * 
- * $Id: score.c,v 1.3.4.2 2001/09/02 07:13:37 rug Exp $
+ * $Id: score.c,v 1.3.4.9 2006/09/17 15:22:24 opengl2772 Exp $
  * 
  * Copyright (C) 2001 Hajime BABA  All rights reserved.
- * Copyright (C) 2001 Namazu Project All rights reserved.
+ * Copyright (C) 2001-2006 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -109,23 +109,24 @@ static int
 get_field_size(int docid, int idxid)
 {
     FILE *fp_field, *fp_field_idx;
-    char fname[BUFSIZE], buf[BUFSIZE];
+    char fname[BUFSIZE] = "", buf[BUFSIZE];
     int size;
     
     /* Make a pathname */
     make_fullpathname_field(idxid);
-    strcpy(fname, NMZ.field);
-    strcat(fname, "size");
+    strncpy(fname, NMZ.field, BUFSIZE - 1);
+    strncat(fname, "size", BUFSIZE - strlen(fname) - 1);
 
     fp_field = fopen(fname, "rb");
     if (fp_field == NULL) {
         nmz_debug_printf("%s: %s", fname, strerror(errno));
         return 0;
     }
-    strcat(fname, ".i");
+    strncat(fname, ".i", BUFSIZE - strlen(fname) - 1);
     fp_field_idx = fopen(fname, "rb");
     if (fp_field_idx == NULL) {
         nmz_warn_printf("%s: %s", fname, strerror(errno));
+        fclose(fp_field_idx);
 	return 0;
     }
 
@@ -146,13 +147,13 @@ static int
 get_field_time(int docid, int idxid)
 {
     FILE *fp_field;
-    char fname[BUFSIZE];
+    char fname[BUFSIZE] = "";
     unsigned int t;
     
     /* Make a pathname */
     make_fullpathname_field(idxid);
-    strcpy(fname, NMZ.t);
-    strcat(fname, "\0");
+    strncpy(fname, NMZ.t, BUFSIZE - 1);
+    strncat(fname, "\0", BUFSIZE - strlen(fname) - 1);
 
     fp_field = fopen(fname, "rb");
     if (fp_field == NULL) {
@@ -173,22 +174,23 @@ static void
 get_field_uri(char *uri, int docid, int idxid)
 {
     FILE *fp_field, *fp_field_idx;
-    char fname[BUFSIZE], buf[BUFSIZE];
+    char fname[BUFSIZE] = "", buf[BUFSIZE];
     
     /* Make a pathname */
     make_fullpathname_field(idxid);
-    strcpy(fname, NMZ.field);
-    strcat(fname, "uri");
+    strncpy(fname, NMZ.field, BUFSIZE - 1);
+    strncat(fname, "uri", BUFSIZE - strlen(fname) - 1);
 
     fp_field = fopen(fname, "rb");
     if (fp_field == NULL) {
         nmz_debug_printf("%s: %s", fname, strerror(errno));
         return;
     }
-    strcat(fname, ".i");
+    strncat(fname, ".i", BUFSIZE - strlen(fname) - 1);
     fp_field_idx = fopen(fname, "rb");
     if (fp_field_idx == NULL) {
         nmz_warn_printf("%s: %s", fname, strerror(errno));
+        fclose(fp_field_idx);
 	return;
     }
 
@@ -429,19 +431,22 @@ nmz_recompute_score(NmzResult *hlist)
     double qds_dis_ratio = 1.0;
     int i;
     
-    orig = malloc(sizeof(int)*(hlist->num + 1));
+    orig = malloc(sizeof(int) * hlist->num);
     if (orig == NULL) {
 	 nmz_set_dyingmsg(nmz_msg("%s", strerror(errno)));
 	 return;
     }
-    qds =  malloc(sizeof(double)*(hlist->num + 1));
+    qds =  malloc(sizeof(double) * hlist->num);
     if (qds == NULL) {
 	 nmz_set_dyingmsg(nmz_msg("%s", strerror(errno)));
+         free(orig);
 	 return;
     }
-    dis =  malloc(sizeof(double)*(hlist->num + 1));
+    dis =  malloc(sizeof(double) * hlist->num);
     if (dis == NULL) {
 	 nmz_set_dyingmsg(nmz_msg("%s", strerror(errno)));
+         free(orig);
+         free(qds);
 	 return;
     }
     
